@@ -1,7 +1,8 @@
 import six
 
 from sevenbridges.meta.resource import Resource
-from sevenbridges.meta.fields import StringField
+from sevenbridges.meta.fields import StringField, CompoundField
+from sevenbridges.models.compound.jobs.job_instance_disk import Disk
 
 
 class Instance(Resource):
@@ -12,6 +13,7 @@ class Instance(Resource):
     id = StringField(read_only=True)
     type = StringField(read_only=True)
     provider = StringField(read_only=True)
+    disk = CompoundField(Disk, read_only=True)
 
     def __str__(self):
         return six.text_type(
