@@ -1,19 +1,23 @@
-import hashlib
 import io
 import os
-import threading
 import time
+import logging
+import hashlib
+import threading
 
-import requests
 import six
+import requests
 
-from sevenbridges.decorators import retry
 from sevenbridges.errors import SbgError
+from sevenbridges.decorators import retry
 from sevenbridges.http.client import generate_session
 from sevenbridges.models.enums import PartSize, TransferState
 from sevenbridges.transfer.utils import Part, Progress, total_parts
 
 
+logger = logging.getLogger(__name__)
+
+
 def _download_part(path, session, url, retry, timeout, start_byte, end_byte):
     """
     Downloads a single part.
@@ -202,7 +206,12 @@ class Download(threading.Thread):
         self._progress_callback = None
         self._time_started = 0
 
-        self._session = generate_session(self._api.session.proxies)
+        self._session = generate_session(
+            self._api.pool_connections,
+            self._api.pool_maxsize,
+            self._api.pool_block,
+            self._api.session.proxies
+        )
 
         try:
             self._file_size = self._get_file_size()
@@ -356,8 +365,11 @@ class Download(threading.Thread):
         self._status = TransferState.COMPLETED
         try:
             os.rename(self._temp_file, self._file_path)
-        except Exception:
-            raise SbgError("Unable to rename the file.")
+        except Exception as e:
+            raise SbgError(
+                "Unable to rename the file due to an error: {}."
+                .format(six.text_type(e))
+            )
 
         if self._callback:
             return self._callback(self._status)
@@ -373,6 +385,6 @@ class Download(threading.Thread):
         )
         file_size = int(file_size)
         if file_size == 0:
-            with io.open(self._file_path, 'a'):
+            with io.open(self._temp_file, 'a', encoding='utf-8'):
                 pass
         return file_size
